@@ -48,6 +48,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <lib/documentview/documentviewcontainer.h>
 #include <lib/documentview/documentviewcontroller.h>
 #include <lib/documentview/documentviewsynchronizer.h>
+#include <lib/fullscreenbar.h>
 #include <lib/gvdebug.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/paintutils.h>
@@ -58,6 +59,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <lib/thumbnailview/thumbnailbarview.h>
 #include <lib/zoomwidget.h>
 #include <lib/zoommode.h>
+#include <lib/stylesheetutils.h>
 
 namespace Gwenview
 {
@@ -73,36 +75,6 @@ namespace Gwenview
 
 const int ViewMainPage::MaxViewCount = 6;
 
-static QString rgba(const QColor &color)
-{
-    return QString::fromLocal8Bit("rgba(%1, %2, %3, %4)")
-           .arg(color.red())
-           .arg(color.green())
-           .arg(color.blue())
-           .arg(color.alpha());
-}
-
-static QString gradient(Qt::Orientation orientation, const QColor &color, int value)
-{
-    int x2, y2;
-    if (orientation == Qt::Horizontal) {
-        x2 = 0;
-        y2 = 1;
-    } else {
-        x2 = 1;
-        y2 = 0;
-    }
-    QString grad =
-        "qlineargradient(x1:0, y1:0, x2:%1, y2:%2,"
-        "stop:0 %3, stop: 1 %4)";
-    return grad
-           .arg(x2)
-           .arg(y2)
-           .arg(rgba(PaintUtils::adjustedHsv(color, 0, 0, qMin(255 - color.value(), value / 2))))
-           .arg(rgba(PaintUtils::adjustedHsv(color, 0, 0, -qMin(color.value(), value / 2))))
-           ;
-}
-
 /*
  * Layout of mThumbnailSplitter is:
  *
@@ -160,9 +132,7 @@ struct ViewMainPagePrivate
     // that use this KPart
     QHash<DocumentView*, KActivities::ResourceInstance*> mActivityResources;
 
-    bool mFullScreenMode;
     bool mCompareMode;
-    bool mThumbnailBarVisibleBeforeFullScreen;
     ZoomMode::Enum mZoomMode;
 
     void setupThumbnailBar()
@@ -181,9 +151,13 @@ struct ViewMainPagePrivate
         Qt::Orientation orientation = mThumbnailBar->orientation();
         QColor bgColor = pal.color(QPalette::Normal, QPalette::Base);
         QColor bgSelColor = pal.color(QPalette::Normal, QPalette::Highlight);
-
+        QColor bgHovColor = pal.color(QPalette::Normal, QPalette::Highlight);
+        
         // Avoid dark and bright colors
         bgColor.setHsv(bgColor.hue(), bgColor.saturation(), (127 + 3 * bgColor.value()) / 4);
+        
+        // Hover uses lighter/faded version of select color. Combine with bgColor to adapt to different backgrounds
+        bgHovColor.setHsv(bgHovColor.hue(), (bgHovColor.saturation() / 2), ((bgHovColor.value() + bgColor.value()) / 2));
 
         QColor leftBorderColor = PaintUtils::adjustedHsv(bgColor, 0, 0, qMin(20, 255 - bgColor.value()));
         QColor rightBorderColor = PaintUtils::adjustedHsv(bgColor, 0, 0, -qMin(40, bgColor.value()));
@@ -196,9 +170,9 @@ struct ViewMainPagePrivate
             "	border-right: 1px solid %3;"
             "}";
         itemCss = itemCss.arg(
-                      gradient(orientation, bgColor, 46),
-                      gradient(orientation, leftBorderColor, 36),
-                      gradient(orientation, rightBorderColor, 26));
+                      StyleSheetUtils::gradient(orientation, bgColor, 46),
+                      StyleSheetUtils::gradient(orientation, leftBorderColor, 36),
+                      StyleSheetUtils::gradient(orientation, rightBorderColor, 26));
 
         QString itemSelCss =
             "QListView::item:selected {"
@@ -207,10 +181,21 @@ struct ViewMainPagePrivate
             "	border-right: 1px solid %2;"
             "}";
         itemSelCss = itemSelCss.arg(
-                         gradient(orientation, bgSelColor, 56),
-                         rgba(borderSelColor));
+                         StyleSheetUtils::gradient(orientation, bgSelColor, 56),
+                         StyleSheetUtils::rgba(borderSelColor));
+        
+        QString itemHovCss =
+            "QListView::item:hover:!selected {"
+            "  background-color: %1;"
+            "  border-left: 1px solid %2;"
+            "  border-right: 1px solid %3;"
+            "}";
+        itemHovCss = itemHovCss.arg(
+                         StyleSheetUtils::gradient(orientation, bgHovColor, 56),
+                         StyleSheetUtils::rgba(leftBorderColor),
+                         StyleSheetUtils::rgba(rightBorderColor));
 
-        QString css = itemCss + itemSelCss;
+        QString css = itemCss + itemSelCss + itemHovCss;
         if (orientation == Qt::Vertical) {
             css.replace("left", "top").replace("right", "bottom");
         }
@@ -400,14 +385,10 @@ ViewMainPage::ViewMainPage(QWidget* parent, SlideShow* slideShow, KActionCollect
     d->mSlideShow = slideShow;
     d->mActionCollection = actionCollection;
     d->mGvCore = gvCore;
-    d->mFullScreenMode = false;
     d->mCompareMode = false;
-    d->mThumbnailBarVisibleBeforeFullScreen = false;
 
     QShortcut* enterKeyShortcut = new QShortcut(Qt::Key_Return, this);
     connect(enterKeyShortcut, &QShortcut::activated, this, &ViewMainPage::slotEnterPressed);
-    QShortcut* escapeKeyShortcut = new QShortcut(Qt::Key_Escape, this);
-    connect(escapeKeyShortcut, &QShortcut::activated, this, &ViewMainPage::slotEscapePressed);
 
     d->setupToolContainer();
     d->setupStatusBar();
@@ -426,7 +407,6 @@ ViewMainPage::ViewMainPage(QWidget* parent, SlideShow* slideShow, KActionCollect
     d->mToggleThumbnailBarAction->setText(i18n("Thumbnail Bar"));
     d->mToggleThumbnailBarAction->setIcon(QIcon::fromTheme("folder-image"));
     actionCollection->setDefaultShortcut(d->mToggleThumbnailBarAction, Qt::CTRL + Qt::Key_B);
-    d->mToggleThumbnailBarAction->setChecked(GwenviewConfig::thumbnailBarIsVisible());
     connect(d->mToggleThumbnailBarAction, &KToggleAction::triggered, this, &ViewMainPage::setThumbnailBarVisibility);
     d->mToggleThumbnailBarButton->setDefaultAction(d->mToggleThumbnailBarAction);
 
@@ -440,6 +420,8 @@ ViewMainPage::ViewMainPage(QWidget* parent, SlideShow* slideShow, KActionCollect
             d->mSynchronizeCheckBox, SLOT(setChecked(bool)));
     connect(d->mSynchronizeCheckBox, SIGNAL(toggled(bool)),
             d->mSynchronizeAction, SLOT(setChecked(bool)));
+
+    installEventFilter(this);
 }
 
 ViewMainPage::~ViewMainPage()
@@ -449,7 +431,7 @@ ViewMainPage::~ViewMainPage()
 
 void ViewMainPage::loadConfig()
 {
-    d->applyPalette(false /* fullScreenMode */);
+    d->applyPalette(window()->isFullScreen());
 
     // FIXME: Not symetric with saveConfig(). Check if it matters.
     Q_FOREACH(DocumentView * view, d->mDocumentViews) {
@@ -460,6 +442,8 @@ void ViewMainPage::loadConfig()
     d->mThumbnailSplitter->setOrientation(orientation == Qt::Horizontal ? Qt::Vertical : Qt::Horizontal);
     d->mThumbnailBar->setOrientation(orientation);
     d->setupThumbnailBarStyleSheet();
+    d->mThumbnailBar->setVisible(GwenviewConfig::thumbnailBarIsVisible());
+    d->mToggleThumbnailBarAction->setChecked(GwenviewConfig::thumbnailBarIsVisible());
 
     int oldRowCount = d->mThumbnailBar->rowCount();
     int newRowCount = GwenviewConfig::thumbnailBarRowCount();
@@ -507,28 +491,15 @@ void ViewMainPage::setStatusBarVisible(bool visible)
 
 void ViewMainPage::setFullScreenMode(bool fullScreenMode)
 {
-    d->mFullScreenMode = fullScreenMode;
-    d->mStatusBarContainer->setVisible(!fullScreenMode);
-
     if (fullScreenMode) {
-        d->mThumbnailBarVisibleBeforeFullScreen = d->mToggleThumbnailBarAction->isChecked();
-        if (d->mThumbnailBarVisibleBeforeFullScreen) {
-            d->mToggleThumbnailBarAction->trigger();
-        }
+        d->mThumbnailBar->setVisible(false);
     } else {
-        if (d->mThumbnailBarVisibleBeforeFullScreen) {
-            d->mToggleThumbnailBarAction->trigger();
-        }
+        d->mThumbnailBar->setVisible(d->mToggleThumbnailBarAction->isChecked());
     }
     d->applyPalette(fullScreenMode);
     d->mToggleThumbnailBarAction->setEnabled(!fullScreenMode);
 }
 
-bool ViewMainPage::isFullScreenMode() const
-{
-    return d->mFullScreenMode;
-}
-
 ThumbnailBarView* ViewMainPage::thumbnailBar() const
 {
     return d->mThumbnailBar;
@@ -577,6 +548,26 @@ QSize ViewMainPage::sizeHint() const
     return QSize(400, 300);
 }
 
+QSize ViewMainPage::minimumSizeHint() const {
+    if (!layout()) {
+        return QSize();
+    }
+
+    QSize minimumSize = layout()->minimumSize();
+    if (window()->isFullScreen()) {
+        // Check minimum width of the overlay fullscreen bar
+        // since there is no layout link which could do this
+        const FullScreenBar* fullScreenBar = findChild<FullScreenBar*>();
+        if (fullScreenBar && fullScreenBar->layout()) {
+            const int fullScreenBarWidth = fullScreenBar->layout()->minimumSize().width();
+            if (fullScreenBarWidth > minimumSize.width()) {
+                minimumSize.setWidth(fullScreenBarWidth);
+            }
+        }
+    }
+    return minimumSize;
+}
+
 QUrl ViewMainPage::url() const
 {
     GV_RETURN_VALUE_IF_FAIL(d->currentView(), QUrl());
@@ -756,20 +747,26 @@ void ViewMainPage::slotEnterPressed()
     emit goToBrowseModeRequested();
 }
 
-void ViewMainPage::slotEscapePressed()
+bool ViewMainPage::eventFilter(QObject* watched, QEvent* event)
 {
-    DocumentView *view = d->currentView();
-    if (view) {
-        AbstractRasterImageViewTool *tool = view->currentTool();
-        if (tool) {
-            QKeyEvent event(QEvent::KeyPress, Qt::Key_Escape, Qt::NoModifier);
-            tool->keyPressEvent(&event);
-            if (event.isAccepted()) {
-                return;
+    if (event->type() == QEvent::ShortcutOverride) {
+        const QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
+        if (keyEvent->key() == Qt::Key_Escape) {
+            const DocumentView* view = d->currentView();
+            if (view) {
+                AbstractRasterImageViewTool* tool = view->currentTool();
+                if (tool) {
+                    QKeyEvent toolKeyEvent(QEvent::KeyPress, Qt::Key_Escape, Qt::NoModifier);
+                    tool->keyPressEvent(&toolKeyEvent);
+                    if (toolKeyEvent.isAccepted()) {
+                        event->accept();
+                    }
+                }
             }
         }
     }
-    emit goToBrowseModeRequested();
+    
+    return QWidget::eventFilter(watched, event);
 }
 
 void ViewMainPage::trashView(DocumentView* view)
