@@ -70,11 +70,10 @@ public:
 
     void setFullScreenMode(bool fullScreen);
 
-    bool isFullScreenMode() const;
-
     int statusBarHeight() const;
 
-    virtual QSize sizeHint() const;
+    QSize sizeHint() const Q_DECL_OVERRIDE;
+    QSize minimumSizeHint() const Q_DECL_OVERRIDE;
 
     /**
      * Returns the url of the current document, or an invalid url if unknown
@@ -141,11 +140,13 @@ private Q_SLOTS:
     void slotViewFocused(DocumentView*);
 
     void slotEnterPressed();
-    void slotEscapePressed();
 
     void trashView(DocumentView*);
     void deselectView(DocumentView*);
 
+protected:
+    bool eventFilter(QObject* watched, QEvent* event) Q_DECL_OVERRIDE;
+
 private:
     friend struct ViewMainPagePrivate;
     ViewMainPagePrivate* const d;
