@@ -21,6 +21,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 // Self
 #include "documentview.h"
 
+// C++ Standard library
+#include <cmath>
+
 // Qt
 #include <QApplication>
 #include <QGraphicsLinearLayout>
@@ -71,6 +74,8 @@ namespace Gwenview
 
 static const qreal REAL_DELTA = 0.001;
 static const qreal MAXIMUM_ZOOM_VALUE = qreal(DocumentView::MaximumZoom);
+static const auto MINSTEP = sqrt(0.5);
+static const auto MAXSTEP = sqrt(2.0);
 
 static const int COMPARE_MARGIN = 4;
 
@@ -95,6 +100,7 @@ struct DocumentViewPrivate
     bool mCurrent;
     bool mCompareMode;
     bool mEraseBorders;
+    int controlWheelAccumulatedDelta;
 
     void setCurrentAdapter(AbstractDocumentViewAdapter* adapter)
     {
@@ -113,8 +119,8 @@ struct DocumentViewPrivate
                              q, SLOT(zoomOut(QPointF)));
             QObject::connect(adapter, SIGNAL(zoomToFitChanged(bool)),
                              q, SIGNAL(zoomToFitChanged(bool)));
-            QObject::connect(adapter, SIGNAL(zoomToFitWidthChanged(bool)),
-                             q, SIGNAL(zoomToFitWidthChanged(bool)));
+            QObject::connect(adapter, SIGNAL(zoomToFillChanged(bool)),
+                             q, SIGNAL(zoomToFillChanged(bool)));
         }
         QObject::connect(adapter, SIGNAL(scrollPosChanged()),
                          q, SIGNAL(positionChanged()));
@@ -136,8 +142,8 @@ struct DocumentViewPrivate
 
         if (mSetup.valid && adapter->canZoom()) {
             adapter->setZoomToFit(mSetup.zoomToFit);
-            adapter->setZoomToFitWidth(mSetup.zoomToFitWidth);
-            if (!mSetup.zoomToFit && !mSetup.zoomToFitWidth) {
+            adapter->setZoomToFill(mSetup.zoomToFill);
+            if (!mSetup.zoomToFit && !mSetup.zoomToFill) {
                 adapter->setZoom(mSetup.zoom);
                 adapter->setScrollPos(mSetup.position);
             }
@@ -147,8 +153,8 @@ struct DocumentViewPrivate
         if (adapter->canZoom()) {
             if (adapter->zoomToFit()) {
                 q->zoomToFitChanged(true);
-            } else if (adapter->zoomToFitWidth()) {
-                q->zoomToFitWidthChanged(true);
+            } else if (adapter->zoomToFill()) {
+                q->zoomToFillChanged(true);
             } else {
                 q->zoomChanged(adapter->zoom());
             }
@@ -242,17 +248,17 @@ struct DocumentViewPrivate
         }
     }
 
-    void uncheckZoomToFitWidth()
+    void uncheckZoomToFill()
     {
-        if (mAdapter->zoomToFitWidth()) {
-            mAdapter->setZoomToFitWidth(false);
+        if (mAdapter->zoomToFill()) {
+            mAdapter->setZoomToFill(false);
         }
     }
 
     void setZoom(qreal zoom, const QPointF& center = QPointF(-1, -1))
     {
         uncheckZoomToFit();
-        uncheckZoomToFitWidth();
+        uncheckZoomToFill();
         zoom = qBound(q->minimumZoom(), zoom, MAXIMUM_ZOOM_VALUE);
         mAdapter->setZoom(zoom, center);
     }
@@ -262,18 +268,17 @@ struct DocumentViewPrivate
         qreal min = q->minimumZoom();
 
         mZoomSnapValues.clear();
-        if (min < 1.) {
-            mZoomSnapValues << min;
-            for (qreal invZoom = 16.; invZoom > 1.; invZoom /= 2.) {
-                qreal zoom = 1. / invZoom;
-                if (zoom > min) {
-                    mZoomSnapValues << zoom;
-                }
-            }
-        }
-        for (qreal zoom = 1; zoom <= MAXIMUM_ZOOM_VALUE ; zoom += 1.) {
+        for (qreal zoom = MINSTEP; zoom > min; zoom *= MINSTEP) {
             mZoomSnapValues << zoom;
         }
+        mZoomSnapValues << min;
+
+        std::reverse(mZoomSnapValues.begin(), mZoomSnapValues.end());
+
+        for (qreal zoom = 1; zoom < MAXIMUM_ZOOM_VALUE; zoom *= MAXSTEP) {
+            mZoomSnapValues << zoom;
+        }
+        mZoomSnapValues << MAXIMUM_ZOOM_VALUE;
 
         q->minimumZoomChanged(min);
     }
@@ -342,6 +347,7 @@ DocumentView::DocumentView(QGraphicsScene* scene)
     d->mCurrent = false;
     d->mCompareMode = false;
     d->mEraseBorders = false;
+    d->controlWheelAccumulatedDelta = 0;
 
     setOpacity(0);
 
@@ -449,12 +455,7 @@ void DocumentView::slotCompleted()
     d->hideLoadingIndicator();
     d->updateCaption();
     d->updateZoomSnapValues();
-    if (!d->mAdapter->zoomToFit()) {
-        qreal min = minimumZoom();
-        if (d->mAdapter->zoom() < min) {
-            d->mAdapter->setZoom(min);
-        }
-    } else if (!d->mAdapter->zoomToFitWidth()) {
+    if (!d->mAdapter->zoomToFit() || !d->mAdapter->zoomToFill()) {
         qreal min = minimumZoom();
         if (d->mAdapter->zoom() < min) {
             d->mAdapter->setZoom(min);
@@ -469,8 +470,8 @@ DocumentView::Setup DocumentView::setup() const
     if (d->mAdapter->canZoom()) {
         setup.valid = true;
         setup.zoomToFit = zoomToFit();
-        setup.zoomToFitWidth = zoomToFitWidth();
-        if (!setup.zoomToFit && !setup.zoomToFitWidth) {
+        setup.zoomToFill = zoomToFill();
+        if (!setup.zoomToFit && !setup.zoomToFill) {
             setup.zoom = zoom();
             setup.position = position();
         }
@@ -502,12 +503,12 @@ void DocumentView::setZoomToFit(bool on)
     d->mAdapter->setZoomToFit(on);
 }
 
-void DocumentView::setZoomToFitWidth(bool on)
+void DocumentView::setZoomToFill(bool on)
 {
-    if (on == d->mAdapter->zoomToFitWidth()) {
+    if (on == d->mAdapter->zoomToFill()) {
         return;
     }
-    d->mAdapter->setZoomToFitWidth(on);
+    d->mAdapter->setZoomToFill(on);
 }
 
 bool DocumentView::zoomToFit() const
@@ -515,15 +516,15 @@ bool DocumentView::zoomToFit() const
     return d->mAdapter->zoomToFit();
 }
 
-bool DocumentView::zoomToFitWidth() const
+bool DocumentView::zoomToFill() const
 {
-    return d->mAdapter->zoomToFitWidth();
+    return d->mAdapter->zoomToFill();
 }
 
 void DocumentView::zoomActualSize()
 {
     d->uncheckZoomToFit();
-    d->uncheckZoomToFitWidth();
+    d->uncheckZoomToFill();
     d->mAdapter->setZoom(1.);
 }
 
@@ -570,24 +571,37 @@ qreal DocumentView::zoom() const
     return d->mAdapter->zoom();
 }
 
+void DocumentView::resizeEvent(QGraphicsSceneResizeEvent *event)
+{
+    d->resizeAdapterWidget();
+    d->updateZoomSnapValues();
+    QGraphicsWidget::resizeEvent(event);
+}
+
 void DocumentView::wheelEvent(QGraphicsSceneWheelEvent* event)
 {
     if (d->mAdapter->canZoom() && event->modifiers() & Qt::ControlModifier) {
+        d->controlWheelAccumulatedDelta += event->delta();
         // Ctrl + wheel => zoom in or out
-        if (event->delta() > 0) {
+        if (d->controlWheelAccumulatedDelta >= QWheelEvent::DefaultDeltasPerStep) {
             zoomIn(event->pos());
-        } else {
+            d->controlWheelAccumulatedDelta = 0;
+        } else if (d->controlWheelAccumulatedDelta <= -QWheelEvent::DefaultDeltasPerStep) {
             zoomOut(event->pos());
+            d->controlWheelAccumulatedDelta = 0;
         }
         return;
     }
     if (GwenviewConfig::mouseWheelBehavior() == MouseWheelBehavior::Browse
         && event->modifiers() == Qt::NoModifier) {
+        d->controlWheelAccumulatedDelta += event->delta();
         // Browse with mouse wheel
-        if (event->delta() > 0) {
+        if (d->controlWheelAccumulatedDelta >= QWheelEvent::DefaultDeltasPerStep) {
             previousImageRequested();
-        } else {
+            d->controlWheelAccumulatedDelta = 0;
+        } else if (d->controlWheelAccumulatedDelta <= -QWheelEvent::DefaultDeltasPerStep) {
             nextImageRequested();
+            d->controlWheelAccumulatedDelta = 0;
         }
         return;
     }
@@ -652,7 +666,6 @@ qreal DocumentView::minimumZoom() const
 void DocumentView::setCompareMode(bool compare)
 {
     d->mCompareMode = compare;
-    d->resizeAdapterWidget();
     if (compare) {
         d->mHud->show();
         d->mHud->setZValue(1);
@@ -714,7 +727,6 @@ void DocumentView::emitFocused()
 void DocumentView::setGeometry(const QRectF& rect)
 {
     QGraphicsWidget::setGeometry(rect);
-    d->resizeAdapterWidget();
     if (d->mBirdEyeView) {
         d->mBirdEyeView->slotZoomOrSizeChanged();
     }
