@@ -54,6 +54,8 @@ struct DocumentViewSynchronizerPrivate
                          q, SLOT(setZoom(qreal)));
         QObject::connect(mCurrentView.data(), SIGNAL(zoomToFitChanged(bool)),
                          q, SLOT(setZoomToFit(bool)));
+        QObject::connect(mCurrentView.data(), SIGNAL(zoomToFillChanged(bool)),
+                         q, SLOT(setZoomToFill(bool)));
         QObject::connect(mCurrentView.data(), SIGNAL(positionChanged()),
                          q, SLOT(updatePosition()));
 
@@ -63,6 +65,7 @@ struct DocumentViewSynchronizerPrivate
             }
             view->setZoom(mCurrentView.data()->zoom());
             view->setZoomToFit(mCurrentView.data()->zoomToFit());
+            view->setZoomToFill(mCurrentView.data()->zoomToFill());
         }
     }
 
@@ -127,6 +130,17 @@ void DocumentViewSynchronizer::setZoomToFit(bool fit)
     d->updateOldPosition();
 }
 
+void DocumentViewSynchronizer::setZoomToFill(bool fit)
+{
+    Q_FOREACH(DocumentView* view, *d->mViews) {
+        if (view == d->mCurrentView.data()) {
+            continue;
+        }
+        view->setZoomToFill(fit);
+    }
+    d->updateOldPosition();
+}
+
 void DocumentViewSynchronizer::updatePosition()
 {
     QPoint pos = d->mCurrentView.data()->position();
